@@ -33,7 +33,8 @@ class PortTypesViewFlag(IntFlag):
     MIDI = 0x02
     CV = 0x04
     VIDEO = 0x08
-    ALL = AUDIO | MIDI | CV | VIDEO
+    ALSA = 0x10
+    ALL = AUDIO | MIDI | CV | VIDEO | ALSA
 
 
 @dataclass
@@ -123,7 +124,7 @@ class GroupPos:
     out_xy: tuple[int, int]
     flags: int = 0
     layout_modes: dict[PortMode, BoxLayoutMode]
-    fully_set: bool = False
+    fully_set: bool = True
     
     def __init__(self):
         self.null_xy = (0, 0)
@@ -290,9 +291,11 @@ class Connection:
         
         if (self.port_out.type is PortType.AUDIO_JACK
                 and self.port_in.type is PortType.AUDIO_JACK):
-            if self.port_out.is_cv() and self.port_in.is_cv():
+            if (self.port_out.subtype is PortSubType.CV
+                    and self.port_in.subtype is PortSubType.CV): 
                 return bool(port_types_view & PortTypesViewFlag.CV)
-            if not self.port_out.is_cv() and not self.port_in.is_cv():
+            if (self.port_out.subtype is PortSubType.REGULAR
+                    and self.port_in.subtype is PortSubType.REGULAR):
                 return bool(port_types_view & PortTypesViewFlag.AUDIO)
         
         return False
@@ -351,6 +354,7 @@ class Port:
     # given by JACK metadatas
     pretty_name = ''
     mdata_portgroup = ''
+    mdata_signal_type = ''
 
     def __init__(self, manager: 'PatchbayManager', port_id: int, name: str,
                  port_type: PortType, flags: int, uuid: int):
@@ -361,7 +365,7 @@ class Port:
         self.flags = flags
         self.uuid = uuid
         self.subtype = PortSubType.REGULAR
-        
+
         if (self.type is PortType.AUDIO_JACK
                 and self.flags & JackPortFlag.IS_CONTROL_VOLTAGE):
             self.subtype = PortSubType.CV
@@ -379,11 +383,12 @@ class Port:
 
     def full_type(self) -> tuple[PortType, PortSubType]:
         return (self.type, self.subtype)
-
-    def is_cv(self) -> bool:
-        return bool(self.flags & JackPortFlag.IS_CONTROL_VOLTAGE)
     
     def short_name(self) -> str:
+        if (self.type is PortType.MIDI_ALSA
+                and self.full_name.startswith((':ALSA_IN:', ':ALSA_OUT:'))):
+            return ':'.join(self.full_name.split(':')[5:])
+        
         if self.full_name.startswith('a2j:'):
             long_name = self.full_name.partition(':')[2]
             if ': ' in long_name:
@@ -457,20 +462,21 @@ class Port:
         patchcanvas.rename_port(
             self.group_id, self.port_id, display_name)
 
+    def select_in_canvas(self):
+        if not self.in_canvas:
+            return
+        
+        patchcanvas.select_port(self.group_id, self.port_id)
+
     def __lt__(self, other: 'Port'):
         if self.type != other.type:
             return (self.type < other.type)
 
-        if (self.flags & JackPortFlag.IS_CONTROL_VOLTAGE
-                !=  other.flags & JackPortFlag.IS_CONTROL_VOLTAGE):
-            return not bool(self.flags & JackPortFlag.IS_CONTROL_VOLTAGE)
+        if self.subtype is not other.subtype:
+            return self.subtype < other.subtype
 
-        if (self.full_name.startswith('a2j:')
-                != other.full_name.startswith('a2j:')):
-            return not self.full_name.startswith('a2j:')
-
-        if self.mode() != other.mode():
-            return (self.mode() < other.mode())
+        # if self.mode() != other.mode():
+        #     return (self.mode() < other.mode())
 
         if self.order is None and other.order is None:
             return self.port_id < other.port_id
@@ -541,18 +547,10 @@ class Portgroup:
 
         self.in_canvas = True
 
-        port_type = self.ports[0].type
-        port_subtype = PortSubType.REGULAR
-        if (port_type is PortType.AUDIO_JACK
-                and self.ports[0].flags & JackPortFlag.IS_CONTROL_VOLTAGE):
-            port_subtype = PortSubType.CV
-        elif (port_type is PortType.MIDI_JACK
-                and self.ports[0].full_name.startswith('a2j:')):
-            port_subtype = PortSubType.A2J
-
-        patchcanvas.add_portgroup(self.group_id, self.portgroup_id,
-                                  self.port_mode, port_type, port_subtype,
-                                  [port.port_id for port in self.ports])
+        patchcanvas.add_portgroup(
+            self.group_id, self.portgroup_id,
+            self.port_mode, self.ports[0].type, self.ports[0].subtype,
+            [port.port_id for port in self.ports])
 
     def remove_from_canvas(self):
         if self.manager.very_fast_operation:
@@ -584,7 +582,10 @@ class Group:
         self.has_gui = False
         self.gui_visible = False
 
-        self._icon_from_metadata = False
+        self.mdata_icon = ''
+
+    def __repr__(self) -> str:
+        return f"Group({self.name})"
 
     def update_ports_in_canvas(self):
         for port in self.ports:
@@ -594,56 +595,30 @@ class Group:
         if self.in_canvas:
             return
 
-        icon_type = BoxType.APPLICATION
-        icon_name = self.name.partition('.')[0].lower()
+        box_type, icon_name = self._get_box_type_and_icon()
 
         do_split = bool(self.current_position.flags & GroupPosFlag.SPLITTED)
         split = BoxSplitMode.YES if do_split else BoxSplitMode.NO
 
-        if self._is_hardware:
-            icon_type = BoxType.HARDWARE
-            if self.a2j_group or self.display_name == "Midi-Bridge":
-                icon_name = "a2j"
-
-        if self.client_icon:
-            icon_type = BoxType.CLIENT
-            icon_name = self.client_icon
-
-        if (self.name.startswith("PulseAudio ")
-                and not self.client_icon):
-            if "sink" in self.name.lower():
-                icon_type = BoxType.MONITOR
-                icon_name = 'monitor_playback'
-            elif "source" in self.name.lower():
-                icon_type = BoxType.MONITOR
-                icon_name = 'monitor_capture'
-
-        elif (self.name.endswith(" Monitor")
-                and not self.client_icon):
-            # this group is (probably) a pipewire Monitor group
-            icon_type = BoxType.MONITOR
-            icon_name = 'monitor_playback'
-
-        self.in_canvas = True
-
         gpos = self.current_position
 
-        self.display_name = self.display_name.replace('.0/', '/')
-        self.display_name = self.display_name.replace('_', ' ')
+        self.display_name = self.display_name.replace('.0/', '/').replace('_', ' ')
         
         display_name = self.name
         if self.manager.use_graceful_names:
             display_name = self.display_name
         
-        layout_modes_ = dict[PortMode, int]()
+        layout_modes_ = dict[PortMode, BoxLayoutMode]()
         for port_mode in (PortMode.INPUT, PortMode.OUTPUT, PortMode.BOTH):
             layout_modes_[port_mode] = gpos.get_layout_mode(port_mode.value)
         
         patchcanvas.add_group(
             self.group_id, display_name, split,
-            icon_type, icon_name, layout_modes=layout_modes_,
+            box_type, icon_name, layout_modes=layout_modes_,
             null_xy=gpos.null_xy, in_xy=gpos.in_xy, out_xy=gpos.out_xy)
 
+        self.in_canvas = True
+
         if do_split:
             gpos.flags |= GroupPosFlag.HAS_BEEN_SPLITTED
             patchcanvas.wrap_group_box(
@@ -687,6 +662,40 @@ class Group:
         
         patchcanvas.rename_group(self.group_id, display_name)
 
+    def _get_box_type_and_icon(self) -> tuple[BoxType, str]:
+        box_type = BoxType.APPLICATION
+        icon_name = self.name.partition('.')[0].lower()
+
+        if self._is_hardware:
+            box_type = BoxType.HARDWARE
+            icon_name = ''
+            if self.a2j_group or self.display_name in ("Midi-Bridge", "a2j"):
+                icon_name = "a2j"
+
+        if self.client_icon:
+            box_type = BoxType.CLIENT
+            icon_name = self.client_icon
+
+        if (self.name.startswith("PulseAudio ")
+                and not self.client_icon):
+            if "sink" in self.name.lower():
+                box_type = BoxType.MONITOR
+                icon_name = 'monitor_playback'
+            elif "source" in self.name.lower():
+                box_type = BoxType.MONITOR
+                icon_name = 'monitor_capture'
+
+        elif (self.name.endswith(" Monitor")
+                and not self.client_icon):
+            # this group is (probably) a pipewire Monitor group
+            box_type = BoxType.MONITOR
+            icon_name = 'monitor_playback'
+        
+        if self.mdata_icon:
+            icon_name = self.mdata_icon
+
+        return (box_type, icon_name)
+
     def semi_hide(self, yesno: bool):
         if not self.in_canvas:
             return 
@@ -738,6 +747,10 @@ class Group:
         if (port_full_name.startswith('a2j:')
                 and not port.flags & JackPortFlag.IS_PHYSICAL):
             port_full_name = port_full_name.partition(':')[2]
+            
+        elif port.type is PortType.MIDI_ALSA:
+            port_full_name = ':'.join(port_full_name.split(':')[4:])
+
         port.display_name = port_full_name.partition(':')[2]
 
         if not self.ports:
@@ -756,6 +769,7 @@ class Group:
 
     def remove_port(self, port: Port):
         if port in self.ports:
+            port.remove_from_canvas()
             self.ports.remove(port)
         
         if self.manager._ports_by_name.get(port.full_name):
@@ -881,21 +895,22 @@ class Group:
 
         patchcanvas.wrap_group_box(self.group_id, port_mode, yesno)
 
-    def set_client_icon(self, icon_name:str, from_metadata=False):
-        # icon from metadata is the prority
-        if not from_metadata and self._icon_from_metadata:
-            return
+    def set_client_icon(self, icon_name: str, from_metadata=False):
+        if from_metadata:
+            self.mdata_icon = icon_name
+        else:
+            self.client_icon = icon_name
+        
+        box_type, ex_icon_name = self._get_box_type_and_icon()
         
-        self._icon_from_metadata = from_metadata
-        self.client_icon = icon_name
         if self.in_canvas:
             patchcanvas.set_group_icon(
-                self.group_id, BoxType.CLIENT, icon_name)
+                self.group_id, box_type, icon_name)
 
     def get_pretty_client(self) -> str:
         for client_name in ('firewire_pcm', 'a2j',
-                            'Hydrogen', 'ardour', 'Ardour', 'Qtractor',
-                            'SooperLooper', 'sooperlooper', 'Luppp',
+                            'Hydrogen', 'ardour', 'Ardour', 'Mixbus', 'mixbus',
+                            'Qtractor', 'SooperLooper', 'sooperlooper', 'Luppp',
                             'seq64', 'calfjackhost', 'rakarrack-plus',
                             'seq192', 'Non-Mixer', 'jack_mixer'):
             if self.name == client_name:
@@ -935,7 +950,7 @@ class Group:
 
             return (name, num)
 
-        def cut_end(name: str, *ends: str)->str:
+        def cut_end(name: str, *ends: str) -> str:
             for end in ends:
                 if name.endswith(end):
                     return name.rsplit(end)[0]
@@ -944,7 +959,10 @@ class Group:
         client_name = self.get_pretty_client()
 
         if (not client_name
-                and port.full_name.startswith(('a2j:', 'Midi-Bridge:'))
+                and ((port.type is PortType.MIDI_JACK
+                        and port.full_name.startswith(
+                            ('a2j:', 'Midi-Bridge:')))
+                     or port.type is PortType.MIDI_ALSA)
                 and port.flags & JackPortFlag.IS_PHYSICAL):
             client_name = 'a2j'
 
@@ -1000,9 +1018,12 @@ class Group:
                     else:
                         display_name += ' ' + num
 
-        elif client_name in ('ardour', 'Ardour'):
+        elif client_name in ('ardour', 'Ardour', 'Mixbus', 'mixbus'):
+            if '/TriggerBox/' in display_name:
+                display_name = '▸ ' + display_name.replace('/TriggerBox/', '/', 1)
+            
             for pt in ('audio', 'midi'):
-                if display_name == "physical_%s_input_monitor_enable" % pt:
+                if display_name == f"physical_{pt}_input_monitor_enable":
                     display_name = "physical monitor"
                     break
             else:
@@ -1072,8 +1093,8 @@ class Group:
                 display_name += " " + num
 
         elif client_name == 'rakarrack-plus':
-            if display_name.startswith('rakarrack-plus '):
-                display_name = display_name.replace('rakarrack-plus ', '', 1)
+            if display_name.startswith(('rakarrack-plus ', 'rakarrack-plus.')):
+                display_name = display_name[15:]
             display_name = display_name.replace('_', ' ')
 
         elif not client_name:
@@ -1131,10 +1152,7 @@ class Group:
             self.remove_from_canvas()
 
     def stereo_detection(self, port: Port) -> Union[Port, None]:
-        if port.type != PortType.AUDIO_JACK:
-            return
-
-        if port.flags & JackPortFlag.IS_CONTROL_VOLTAGE:
+        if port.type != PortType.AUDIO_JACK or port.subtype != PortSubType.REGULAR:
             return
 
         # find the last port with same type and mode in the group
@@ -1143,8 +1161,8 @@ class Group:
                 continue
 
             if (other_port.type == port.type
+                    and other_port.subtype == port.subtype
                     and other_port.mode() == port.mode()
-                    and not other_port.flags & JackPortFlag.IS_CONTROL_VOLTAGE
                     and not other_port.portgroup_id
                     and not other_port.prevent_stereo):
                 for portgroup_mem in self.manager.portgroups_memory:
@@ -1430,9 +1448,9 @@ class Group:
                     else:
                         portgroups_mdata.append(
                             {'pg_name': port.mdata_portgroup,
-                            'port_type': port.type,
-                            'port_mode': port.mode(),
-                            'ports':[port]})
+                             'port_type': port.type,
+                             'port_mode': port.mode(),
+                             'ports':[port]})
         
         for pg_mdata in portgroups_mdata:
             if len(pg_mdata['ports']) < 2:
