@@ -5,6 +5,7 @@ import operator
 from pathlib import Path
 from dataclasses import dataclass
 from typing import TYPE_CHECKING, Any, Callable, Optional, Union
+from unittest.mock import patch
 
 from PyQt5.QtGui import QCursor, QGuiApplication
 from PyQt5.QtWidgets import QMainWindow, QMessageBox, QWidget
@@ -63,7 +64,6 @@ def later_by_batch(draw_group=False, sort_group=False, clear_conns=False):
         def wrapper(*args, **kwargs):
             mng = args[0]
             assert isinstance(mng, PatchbayManager)
-            
             if mng.very_fast_operation:
                 return func(*args, **kwargs)
 
@@ -128,6 +128,8 @@ class PatchbayManager:
         
         self._manual_path: Path = None
 
+        self.server_is_started = True
+
         self.sg = SignalsObject()
 
         self._next_group_id = 0
@@ -135,11 +137,12 @@ class PatchbayManager:
         self._next_connection_id = 0
         self._next_portgroup_id = 1
 
-        self.default_zoom = settings.value('Canvas/default_zoom', 100, type=int)
         self.set_graceful_names(settings.value(
             'Canvas/use_graceful_names', True, type=bool))
         self.group_a2j_hw = settings.value(
             'Canvas/group_a2j_ports', True, type=bool)
+        self.alsa_midi_enabled = settings.value(
+            'Canvas/alsa_midi_enabled', False, type=bool)
 
         # all patchbay events are delayed
         # to reduce the patchbay comsumption.
@@ -159,6 +162,7 @@ class PatchbayManager:
         
         self.sg.graceful_names_changed.connect(self.set_graceful_names)
         self.sg.a2j_grouped_changed.connect(self.set_a2j_grouped)
+        self.sg.alsa_midi_enabled_changed.connect(self.set_alsa_midi_enabled)
         self.sg.group_shadows_changed.connect(self.set_group_shadows)
         self.sg.auto_select_items_changed.connect(self.set_auto_select_items)
         self.sg.elastic_changed.connect(self.set_elastic_canvas)
@@ -221,6 +225,25 @@ class PatchbayManager:
         patchcanvas.init(view, self.__canvas_callback__, theme_paths, default_theme_name)
         patchcanvas.canvas.scene.scale_changed.connect(self._scene_scale_changed)
         
+        # just to have the zoom slider updated with the default zoom
+        patchcanvas.canvas.scene.zoom_reset()
+
+        # get port_types_view from config file only if the port type view
+        # filter widget is visible.         
+        try:
+            port_types_view = PortTypesViewFlag(
+                self._settings.value(
+                    'Canvas/default_port_types_view',
+                    PortTypesViewFlag.ALL.value,
+                    type=int))
+        except:
+            _logger.warning(
+                "unable to find correct default port types view "
+                "in the config file.")
+            port_types_view = PortTypesViewFlag.ALL
+                
+        self.change_port_types_view(port_types_view)
+        
     def _scene_scale_changed(self, value: float):
         self.sg.scene_scale_changed.emit(value)
 
@@ -307,6 +330,11 @@ class PatchbayManager:
             elif sub_type is (PortSubType.REGULAR | PortSubType.CV):
                 return bool(self.port_types_view & (PortTypesViewFlag.AUDIO
                                                     | PortTypesViewFlag.CV))
+        if port_type is PortType.MIDI_ALSA:
+            return bool(self.alsa_midi_enabled
+                        and self.port_types_view & PortTypesViewFlag.ALSA)
+        if port_type is PortType.VIDEO:
+            return bool(self.port_types_view & PortTypesViewFlag.VIDEO)
 
         return False
 
@@ -439,8 +467,8 @@ class PatchbayManager:
         self._next_portgroup_id = 1
         self._next_connection_id = 0
 
-    def change_port_types_view(self, port_types_view: PortTypesViewFlag):
-        if port_types_view is self.port_types_view:
+    def change_port_types_view(self, port_types_view: PortTypesViewFlag, force=False):
+        if not force and port_types_view is self.port_types_view:
             return
         
         self.port_types_view = port_types_view
@@ -484,6 +512,11 @@ class PatchbayManager:
             self.group_a2j_hw = bool(yesno)
             self.refresh()
 
+    def set_alsa_midi_enabled(self, yesno: int):
+        if self.alsa_midi_enabled != bool(yesno):
+            self.alsa_midi_enabled = bool(yesno)
+            self.change_port_types_view(PortTypesViewFlag.ALL, force=True)
+
     def set_group_shadows(self, yesno: int):
         patchcanvas.options.show_shadows = bool(yesno)
         self.remove_and_add_all()
@@ -541,6 +574,10 @@ class PatchbayManager:
             port_type = PortType.AUDIO_JACK
         elif port_type_int == PortType.MIDI_JACK:
             port_type = PortType.MIDI_JACK
+        elif port_type_int == PortType.MIDI_ALSA:
+            port_type = PortType.MIDI_ALSA
+        elif port_type_int == PortType.VIDEO:
+            port_type = PortType.VIDEO
 
         port = Port(self, self._next_port_id, name, port_type, flags, uuid)
         self._next_port_id += 1
@@ -551,7 +588,16 @@ class PatchbayManager:
         is_a2j_group = False
         group_is_new = False
 
-        if (full_port_name.startswith(('a2j:', 'Midi-Bridge:'))
+        if (port_type is PortType.MIDI_ALSA
+                and full_port_name.startswith((':ALSA_OUT:', ':ALSA_IN:'))):
+            _, _alsa_key, alsa_gp_id, alsa_p_id, group_name, *rest = \
+                full_port_name.split(':')
+            port_name = ':'.join(rest)
+
+            if port.flags & JackPortFlag.IS_PHYSICAL:
+                is_a2j_group = True
+
+        elif (full_port_name.startswith(('a2j:', 'Midi-Bridge:'))
                 and (not self.group_a2j_hw
                      or not port.flags & JackPortFlag.IS_PHYSICAL)):
             group_name, colon, port_name = port_name.partition(':')
@@ -760,7 +806,6 @@ class PatchbayManager:
     def remove_connection(self, port_out_name: str, port_in_name: str):
         port_out = self.get_port_from_name(port_out_name)
         port_in = self.get_port_from_name(port_in_name)
-
         if port_out is None or port_in is None:
             return
 
@@ -776,21 +821,31 @@ class PatchbayManager:
 
     @in_main_thread()
     def server_started(self):
+        self.server_is_started = True
         if self._tools_widget is not None:
             self._tools_widget.set_jack_running(True)
 
     @in_main_thread()
     def server_stopped(self):
+        self.server_is_started = False
         if self._tools_widget is not None:
-            self._tools_widget.set_jack_running(False)
+            self._tools_widget.set_jack_running(
+                False, use_alsa_midi=self.alsa_midi_enabled)
+
         self.clear_all()
+        if self.alsa_midi_enabled:
+            self.refresh()
 
     @in_main_thread()
     def server_lose(self):
+        self.server_is_started = False
+        
         if self._tools_widget is not None:
             self._tools_widget.set_jack_running(False)
 
         self.clear_all()
+        if self.alsa_midi_enabled:
+            self.refresh()
 
         if self.main_win is not None:
             ret = QMessageBox.critical(
@@ -958,7 +1013,12 @@ class PatchbayManager:
         gps_and_ports = list[tuple[str, list[Port]]]()
         for group in self.groups:
             for port in group.ports:
-                group_name = port.full_name.partition(':')[0]
+                if port.type is PortType.MIDI_ALSA:
+                    group_name = port.full_name.split(':')[4]
+                    print('goreupp name', group_name)
+                else:
+                    group_name = port.full_name.partition(':')[0]
+
                 for gp_name, gp_port_list in gps_and_ports:
                     if gp_name == group_name:
                         gp_port_list.append(port)
@@ -1018,6 +1078,8 @@ class PatchbayManager:
                             contents += ':AUDIO'
                     elif port.type is PortType.MIDI_JACK:
                         contents += ':MIDI'
+                    elif port.type is PortType.MIDI_ALSA:
+                        contents += ':ALSA'
 
                     contents += f':{port.mode().name}\n'
                     last_type_and_mode = (port.type, port.mode())
@@ -1035,7 +1097,10 @@ class PatchbayManager:
                         contents += ':~PORTGROUP\n'
                     pg_name = port.mdata_portgroup
 
-                port_short_name = port.full_name.partition(':')[2]
+                if port.type is PortType.MIDI_ALSA:
+                    port_short_name = ':'.join(port.full_name.split(':')[5:])
+                else:
+                    port_short_name = port.full_name.partition(':')[2]
                 contents += f'{port_short_name}\n'
                 
                 if port.pretty_name or port.order:
