@@ -32,9 +32,11 @@ from PyQt5.QtGui import QCursor, QPixmap, QPolygonF, QBrush
 from PyQt5.QtWidgets import (QGraphicsRectItem, QGraphicsScene, QApplication,
                              QGraphicsView, QGraphicsItem)
 
+
 # Imports (locals)
 from .init_values import (
     CanvasItemType,
+    PortMode,
     Direction,
     canvas,
     options,
@@ -484,6 +486,8 @@ class PatchSceneMoth(QGraphicsScene):
         full_rect = QRectF()
 
         for widget in canvas.list_boxes():
+            if widget.get_current_port_mode() is PortMode.NULL:
+                continue
             full_rect |= widget.sceneBoundingRect().marginsAdded(
                 QMarginsF(50.0, 20.0, 50.0, 20.0))
 
@@ -529,11 +533,11 @@ class PatchSceneMoth(QGraphicsScene):
         
         self._view.viewport().unsetCursor()
 
-    def zoom_ratio(self, percent: float):
+    def zoom_ratio(self, percent: float, force=False):
         ratio = percent / 100.0
         transform = self._view.transform()
         
-        if ratio == transform.m11():
+        if not force and ratio == transform.m11():
             return
         
         transform.reset()
@@ -597,7 +601,7 @@ class PatchSceneMoth(QGraphicsScene):
     def zoom_reset(self):
         transform = self._view.transform()
         transform.reset()
-        
+
         default_scale = options.default_zoom / 100
         transform.scale(default_scale, default_scale)
         self._view.setTransform(transform)
