@@ -1,7 +1,7 @@
 #!/usr/bin/python3 -u
 
 APP_TITLE = 'Patchance'
-VERSION = (1, 0, 0)
+VERSION = (1, 1, 0)
 
 import sys
 
@@ -23,19 +23,31 @@ for arg in sys.argv[1:]:
         sys.exit(0)
 
 
+from typing import Optional
 import signal
 import logging
 
 from dataclasses import dataclass
 from os.path import dirname
 
+
 from PyQt5.QtWidgets import QApplication
 from PyQt5.QtGui import QIcon, QFontDatabase
 from PyQt5.QtCore import QLocale, QTranslator, QTimer, QLibraryInfo, QSettings
 
+try:
+    from pyalsa.alsaseq import SEQ_LIB_VERSION_STR
+    ALSA_VERSION_LIST = [int(num) for num in SEQ_LIB_VERSION_STR.split('.')]
+    assert ALSA_VERSION_LIST >= [1, 2, 4]
+    ALSA_LIB_OK = True
+except:
+    ALSA_LIB_OK = False
+
 from main_win import MainWindow
 from patchance_pb_manager import PatchancePatchbayManager
 from jack_manager import JackManager
+if ALSA_LIB_OK:
+    from alsa_manager import AlsaManager
 
 
 @dataclass
@@ -44,6 +56,7 @@ class Main:
     main_win: MainWindow
     patchbay_manager: PatchancePatchbayManager
     jack_manager: JackManager
+    alsa_manager: 'Optional[AlsaManager]'
     settings: QSettings
 
 
@@ -63,6 +76,8 @@ def make_logger():
     logger.addHandler(log_handler)
 
 def main_loop():
+    global main
+    
     make_logger()
     
     import resources_rc
@@ -107,14 +122,32 @@ def main_loop():
     main_win = MainWindow()
     pb_manager = PatchancePatchbayManager(settings)
     jack_manager = JackManager(pb_manager)
+    
+    if ALSA_LIB_OK:
+        alsa_manager = AlsaManager(pb_manager)
+        if settings.value('Canvas/alsa_midi_enabled', False, type=bool):
+            alsa_manager.add_all_ports()
+    else:
+        alsa_manager = None
+
+    main = Main(app,
+                main_win,
+                pb_manager,
+                jack_manager,
+                alsa_manager,
+                settings)
 
-    main = Main(app, main_win, pb_manager, jack_manager, settings)
     pb_manager.finish_init(main)
+    if not ALSA_LIB_OK:
+        pb_manager.options_dialog.enable_alsa_midi(False)
+    
     main_win.finish_init(main)
     main_win.show()
 
     app.exec()
     settings.sync()
+    if alsa_manager is not None:
+        alsa_manager.stop_events_loop()
     jack_manager.exit()
     pb_manager.save_positions()
     del app
