@@ -13,6 +13,7 @@ from patchbay import (
     Callbacker,
     CanvasOptionsDialog,
     PatchbayManager)
+from patchbay.patchcanvas.init_values import PortType
 
 from tools import get_code_root
 import xdg
@@ -40,6 +41,11 @@ class PatchanceCallbacker(Callbacker):
         if port_out is None or port_in is None:
             return
         
+        if port_out.type is PortType.MIDI_ALSA:
+            if self.mng.alsa_mng is None:
+                return
+        
+            self.mng.alsa_mng.connect_ports(port_out.full_name, port_in.full_name)
         if self.mng.jack_mng is None:
             return
         
@@ -48,9 +54,19 @@ class PatchanceCallbacker(Callbacker):
     def _ports_disconnect(self, connection_id: int):
         for conn in self.mng.connections:
             if conn.connection_id == connection_id:
+                if conn.port_type() is PortType.MIDI_ALSA:
+                    if self.mng.alsa_mng is None:
+                        return
+                    
+                    self.mng.alsa_mng.connect_ports(
+                        conn.port_out.full_name,
+                        conn.port_in.full_name,
+                        disconnect=True)
+                    return
+
                 self.mng.jack_mng.disconnect_ports(
                     conn.port_out.full_name, conn.port_in.full_name)
-                break
+                return
 
 
 class PatchancePatchbayManager(PatchbayManager):
@@ -59,6 +75,7 @@ class PatchancePatchbayManager(PatchbayManager):
         self._settings = settings
         
         self.jack_mng = None
+        self.alsa_mng = None
         self._memory_path = None
         
         if settings is not None:
@@ -123,6 +140,9 @@ class PatchancePatchbayManager(PatchbayManager):
         super().refresh()
         if self.jack_mng is not None:
             self.jack_mng.init_the_graph()
+        
+        if self.alsa_mng is not None:
+            self.alsa_mng.add_all_ports()
     
     def change_buffersize(self, buffer_size: int):
         super().change_buffersize(buffer_size)
@@ -140,8 +160,18 @@ class PatchancePatchbayManager(PatchbayManager):
     def transport_relocate(self, frame: int):
         self.jack_mng.transport_relocate(frame)
 
+    def set_alsa_midi_enabled(self, yesno: int):
+        if self.alsa_mng is not None:    
+            if yesno:
+                self.alsa_mng.add_all_ports()
+            else:
+                self.alsa_mng.stop_events_loop()
+        
+        super().set_alsa_midi_enabled(yesno)
+
     def finish_init(self, main: 'Main'):
         self.jack_mng = main.jack_manager
+        self.alsa_mng = main.alsa_manager
         self.set_main_win(main.main_win)
         self._setup_canvas()
 
